#!/bin/sh
set -x
printf "\n\n" > milkytracker.help
lynx -dump ./docs/MilkyTracker.html | grep -v "file:" | awk '{ print "  "$0"\n" }' | sed 's/Example:/\n    Example:\n\n\n    /g;s/Explanation:/\n\n     Explanation:/g;s/·/\./g' >> milkytracker.help
{
  echo "// generated by src/tools/generateHelp.sh"
  xxd -i milkytracker.help 
} > src/tracker/DialogHelpText.h
